@@ -219,6 +219,7 @@ class PreferencesItem(Gtk.ImageMenuItem):
         self.connect('activate', self.launch_preferences_dialog)
 
     def launch_preferences_dialog(self, _item):
+        # pylint: disable=consider-using-with
         subprocess.Popen(['qubes-vm-settings', self.vm.name])
 
 
@@ -236,6 +237,7 @@ class LogItem(Gtk.ImageMenuItem):
         self.connect('activate', self.launch_log_viewer)
 
     def launch_log_viewer(self, *_args, **_kwargs):
+        # pylint: disable=consider-using-with
         subprocess.Popen(['qubes-log-viewer', self.path])
 
 
@@ -295,12 +297,13 @@ class StartedMenu(Gtk.Menu):
         self.vm = vm
         self.app = app
 
+        self.add(OpenFileManagerItem(self.vm, icon_cache))
+        self.add(RunTerminalItem(self.vm, icon_cache))
         self.add(PreferencesItem(self.vm, icon_cache))
         self.add(PauseItem(self.vm, icon_cache))
         self.add(ShutdownItem(self.vm, self.app, icon_cache))
-        self.add(RestartItem(self.vm, self.app, icon_cache))
-        self.add(RunTerminalItem(self.vm, icon_cache))
-        self.add(OpenFileManagerItem(self.vm, icon_cache))
+        if self.vm.klass != 'DispVM' or not self.vm.auto_cleanup:
+            self.add(RestartItem(self.vm, self.app, icon_cache))
 
         self.show_all()
 
@@ -315,7 +318,6 @@ class PausedMenu(Gtk.Menu):
         self.add(PreferencesItem(self.vm, icon_cache))
         self.add(UnpauseItem(self.vm, icon_cache))
         self.add(KillItem(self.vm, icon_cache))
-        self.add(RunTerminalItem(self.vm, icon_cache))
 
         self.show_all()
 
@@ -346,6 +348,7 @@ class DebugMenu(Gtk.Menu):
 
 
 def run_manager(_item):
+    # pylint: disable=consider-using-with
     subprocess.Popen(['qubes-qube-manager'])
 
 
@@ -510,6 +513,7 @@ class DomainTray(Gtk.Application):
         self.register()  # register Gtk Application
 
     def register_events(self):
+        self.dispatcher.add_handler('connection-established', self.refresh_all)
         self.dispatcher.add_handler('domain-pre-start', self.update_domain_item)
         self.dispatcher.add_handler('domain-start', self.update_domain_item)
         self.dispatcher.add_handler('domain-start-failed',
@@ -802,10 +806,19 @@ class DomainTray(Gtk.Application):
 
         self.connect('shutdown', self._disconnect_signals)
 
+    def refresh_all(self, _subject, _event, **_kwargs):
+        for vm in self.menu_items:
+            if vm not in self.qapp.domains:
+                self.remove_domain_item(None, None, vm)
+        for vm in self.qapp.domains:
+            self.update_domain_item(vm, '')
+
     def run(self):  # pylint: disable=arguments-differ
         self.initialize_menu()
 
     def _disconnect_signals(self, _event):
+        self.dispatcher.remove_handler('connection-established',
+                                       self.refresh_all)
         self.dispatcher.remove_handler('domain-pre-start',
                                        self.update_domain_item)
         self.dispatcher.remove_handler('domain-start', self.update_domain_item)
